@@ -412,6 +412,14 @@ void fix_to_struct2(MathStructure &m) {
 MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo, bool always_convert, bool convert_to_mixed_units) {
 	return convert(mstruct, to_unit, eo, always_convert, convert_to_mixed_units, false, NULL);
 }
+Unit *find_ounce(const MathStructure &m) {
+	if(m.isUnit() && m.unit()->referenceName() == "oz") return m.unit();
+	for(size_t i = 0; i < m.size(); i++) {
+		Unit *u = find_ounce(m[i]);
+		if(u) return u;
+	}
+	return NULL;
+}
 MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo, bool always_convert, bool convert_to_mixed_units, bool transform_orig, MathStructure *parsed_struct) {
 	CompositeUnit *cu = NULL;
 	if(to_unit->subtype() == SUBTYPE_COMPOSITE_UNIT) cu = (CompositeUnit*) to_unit;
@@ -421,7 +429,9 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 	if(!b_ratio && to_unit->baseUnit() != getRadUnit() && !mstruct.containsType(STRUCT_UNIT, true)) {
 		if(transform_orig && (!parsed_struct || !parsed_struct->containsType(STRUCT_UNIT, false, true, true))) {
 			// multiply original value with base units
-			MathStructure munit(convertToOptimalUnit(to_unit, eo, true));
+			EvaluationOptions eo2 = eo;
+			if(eo.approximation == APPROXIMATION_EXACT) eo2.approximation = APPROXIMATION_TRY_EXACT;
+			MathStructure munit(convertToOptimalUnit(to_unit, eo2, true));
 			munit.unformat();
 			fix_to_struct(munit);
 			if(!munit.isZero()) {
@@ -494,7 +504,9 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 			// 1 = J, 2 = K, 3 = 1/m, 4 = Hz
 			int u1_type = 0;
 			int exp = to_unit->baseExponent();
-			if(exp == 1) {
+			if(to_unit->referenceName() == "oz") {
+				u1_type = 6;
+			} else if(exp == 1) {
 				if(bu->subtype() == SUBTYPE_COMPOSITE_UNIT) {
 					CompositeUnit *cu2 = (CompositeUnit*) bu;
 					if(cu2->countUnits() == 1) {
@@ -509,6 +521,25 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 					} else if(cu2->countUnits() == 2 && cu2->referenceName() == "N_m") {
 						u1_type = 1;
 					}
+					if(u1_type == 0) {
+						for(size_t i = 1; i <= cu2->countUnits(); i++) {
+							if(to_unit == cu2 && cu2->get(i)->referenceName() == "oz" && !to_unit->isRegistered()) {
+								u1_type = 6;
+								break;
+							}
+						}
+						for(size_t i = 1; i <= cu2->countUnits(); i++) {
+							bu = cu2->get(i, &exp)->baseUnit();
+							exp *= bu->baseExponent();
+							if((u1_type == 0 || u1_type == 6) && exp % 3 == 0 && bu->referenceName() == "m") {
+								if(u1_type == 6) {u1_type = 0; break;}
+								u1_type = 5;
+							} else if(u1_type != 6 && bu->referenceName() == "g") {
+								u1_type = 0;
+								break;
+							}
+						}
+					}
 				} else if(bu->referenceName() == "K") {
 					u1_type = 2;
 				}
@@ -518,19 +549,38 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 				} else if(bu->referenceName() == "s") {
 					u1_type = 4;
 				}
+			} else if(exp % 3 == 0) {
+				if(bu->referenceName() == "m") {
+					u1_type = 5;
+				}
 			}
-			if(u1_type > 0) {
+			if(u1_type == 5) {
+				Unit *u1 = find_ounce(mstruct_new);
+				if(u1 && (!parsed_struct || find_ounce(*parsed_struct))) {
+					Unit *u_gram = getActiveUnit("g");
+					if(!to_unit->containsRelativeTo(u_gram)) {
+						Unit *u2 = getActiveUnit("fl_oz");
+						if(u2) {
+							if(parsed_struct) parsed_struct->replace(u1, u2);
+							mstruct_new.replace(u1, u2);
+						}
+					}
+				}
+			} else if(u1_type > 0) {
 				const MathStructure *mstruct_u = NULL;
 				if(mstruct_new.isUnit_exp()) {
 					mstruct_u = &mstruct_new;
 				} else if(mstruct_new.isMultiplication() && mstruct_new.size() >= 2 && mstruct_new.last().isUnit_exp() && !mstruct_new[mstruct_new.size() - 2].isUnit_exp()) {
 					mstruct_u = &mstruct_new.last();
 				}
-				if(mstruct_u && (!mstruct_u->isPower() || (u1_type == 1 && (*mstruct_u)[1].isMinusOne()))) {
+				if(mstruct_u && (!mstruct_u->isPower() || (u1_type == 6 && (*mstruct_u)[1].isInteger()) || (u1_type == 1 && (*mstruct_u)[1].isMinusOne()))) {
 					if(mstruct_u->isPower()) bu = (*mstruct_u)[0].unit();
 					else bu = mstruct_u->unit();
 					exp = bu->baseExponent();
-					if(mstruct_u->isPower()) exp = -exp;
+					if(mstruct_u->isPower()) {
+						if(u1_type == 6) exp *= (*mstruct_u)[1].number().intValue();
+						else exp = -exp;
+					}
 					bu = bu->baseUnit();
 					int u2_type = 0;
 					if(exp == 1) {
@@ -548,6 +598,19 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 							} else if(u1_type != 1 && cu2->countUnits() == 2 && cu2->referenceName() == "N_m") {
 								u2_type = 1;
 							}
+							if(u2_type == 0 && u1_type == 6) {
+								for(size_t i = 1; i <= cu2->countUnits(); i++) {
+									bu = cu2->get(i, &exp);
+									exp *= bu->baseExponent();
+									bu = bu->baseUnit();
+									if(u2_type == 0 && exp % 3 == 0 && bu->referenceName() == "m") {
+										u2_type = 5;
+									} else if(bu->referenceName() == "g") {
+										u2_type = 0;
+										break;
+									}
+								}
+							}
 						} else if(u1_type == 1 && bu->referenceName() == "K") {
 							u2_type = 2;
 						}
@@ -557,6 +620,8 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 						} else if(bu->referenceName() == "s") {
 							u2_type = 4;
 						}
+					} else if(exp % 3 == 0 && u1_type == 6 && bu->referenceName() == "m") {
+						u2_type = 5;
 					}
 					Variable *v = NULL;
 					if(u1_type == 1) {
@@ -567,6 +632,23 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 						if(u1_type == 2) v = getActiveVariable("J_to_K");
 						else if(u1_type == 3) v = getActiveVariable("J_to_m");
 						else if(u1_type == 4) v = getActiveVariable("J_to_Hz");
+					} else if(u1_type == 6 && u2_type == 5) {
+						Unit *u = getActiveUnit("fl_oz");
+						if(u) {
+							if(to_unit->subtype() == SUBTYPE_COMPOSITE_UNIT) {
+								CompositeUnit *cu2 = (CompositeUnit*) to_unit;
+								for(size_t i = 1; i <= cu2->countUnits(); i++) {
+									bu = cu2->get(i, &exp);
+									if(bu->referenceName() == "oz") {
+										cu2->del(i);
+										cu2->add(u, exp);
+										break;
+									}
+								}
+							} else {
+								to_unit = u;
+							}
+						}
 					}
 					if(v) {
 						mstruct_new *= v;
@@ -1567,15 +1649,15 @@ MathStructure Calculator::convert(const MathStructure &mstruct_to_convert, strin
 	}
 	MathStructure mstruct;
 	bool b = false;
-	Unit *u = getUnit(str2);
+	Unit *u = getActiveUnit(str2);
 	if(!u) u = getCompositeUnit(str2);
 	Variable *v = NULL;
-	if(!u) v = getVariable(str2);
+	if(!u) v = getActiveVariable(str2);
 	if(!u && !v) {
 		for(size_t i = 0; i < signs.size(); i++) {
 			if(str2 == signs[i]) {
 				u = getUnit(real_signs[i]);
-				if(!u) v = getVariable(real_signs[i]);
+				if(!u) v = getActiveVariable(real_signs[i]);
 				break;
 			}
 		}
